@@ -553,6 +553,13 @@ function templates(tpl,key) {
                   "title" : "oidcRPMetaDataOptionsAllowPasswordGrant",
                   "type" : "bool"
                },
+               {
+                  "default" : 0,
+                  "get" : tpl+"s/"+key+"/"+"oidcRPMetaDataOptionsAllowClientCredentialsGrant",
+                  "id" : tpl+"s/"+key+"/"+"oidcRPMetaDataOptionsAllowClientCredentialsGrant",
+                  "title" : "oidcRPMetaDataOptionsAllowClientCredentialsGrant",
+                  "type" : "bool"
+               },
                {
                   "get" : tpl+"s/"+key+"/"+"oidcRPMetaDataOptionsAuthnLevel",
                   "id" : tpl+"s/"+key+"/"+"oidcRPMetaDataOptionsAuthnLevel",
